"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const databases_1 = require("./utils/databases");
const Fastify = __importStar(require("fastify"));
const elastic_apm_node_1 = __importDefault(require("elastic-apm-node"));
class Microservice {
    constructor(config) {
        this.agent = elastic_apm_node_1.default.start({
            logLevel: 'trace'
        });
        if (config.databases) {
            databases_1.initDbInstances(this, config.databases);
        }
        //init MicroService
        this.server = Fastify.fastify({
            logger: true
        });
        this.config = config;
        this.server.get('/', (request, reply) => {
            reply.send({ hello: 'world' });
        });
        this.server.get('/fail', (request, reply) => {
            const err = new Error('test error');
            this.agent.captureError(err);
            reply.send({ hello: 'fail' });
        });
    }
    declareRoute(verb, path, options, handler) {
        let { onRequest } = options;
        const apmContextInject = ((req) => this.agent.setCustomContext(req));
        if (onRequest) {
            onRequest = typeof onRequest === 'function' ? [apmContextInject, onRequest] : [apmContextInject, ...onRequest];
        }
        else {
            onRequest = [apmContextInject];
        }
        this.server.route({
            ...options,
            method: verb,
            url: path,
            handler: handler,
            onRequest: onRequest
        });
    }
    log(log) {
        console.log(log);
    }
    errorLog(log, error) {
        console.log(`ERROR [${error}] - error`);
    }
    async start() {
        return await this.server
            .listen(this.config.port)
            .then(() => {
            console.log('SERVER LISTENING');
            return true;
        })
            .catch((err) => {
            console.log(err);
            return false;
        });
    }
}
exports.default = Microservice;
